%% Copyright 2012 Soren Hauberg
%%
%% This program is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program.  If not, see <http://www.gnu.org/licenses/>.

function y = local_diagonal_gaussian_geodesic_system (state, manifold)
  %% Basic info
  mu = [manifold.mu{:}]; % size: DxR
  M = [manifold.M{:}]; % size: DxR
  lambda = manifold.lambda;
  [D, R] = size (mu);
  
  %% Create index structure for outer products along one dimension
  outidx_left  = repmat (1:D, 1, D);
  outidx_right = repmat (1:D, D, 1);
  
  %% Interpret state
  [~, N] = size (state);
  c = state (1:D, :); % point on curve, size: DxN
  cm = state ((D+1):end, :); % curve derivative, size: DxN

  %% Compute weighted metric tensors in the points on the curve
  [Ms, ~, Wt] = metric_tensor (c, manifold); % sizes: Ms: DxN, Wt: RxN

  %% 
  muM = mu .* M; % size: DxR
  P = repmat (c (:), 1, R) .* repmat (M, N, 1) - repmat (muM, N, 1); % size: DNxR
  P (:, end) = 0; % XXX: Only have this when you also have a fixed tensor in 'metric_tensor'
 
  dMdc = zeros (D^2, N);
  bla = Wt (:, repmat (1:N, D, 1)).'; % DNxR
  for r = 1:R
    mrd   = repmat (M (:, r), 1, N); % DxN
    wtr   = repmat (Wt (r, :), D, 1); % DxN

    delta = repmat (P (:, r), 1, R) - P; % DNxR
    sq    = sum (delta .* bla, 2); % DNx1
    sq    = reshape (sq, D, N); % DxN

    incr  = wtr .* sq; % DxN
    tmp = mrd (outidx_left, :) .* incr (outidx_right, :);
    dMdc  = dMdc + tmp;
  end % for
  dMdc = lambda * dMdc;

  %% Compute dMdc * (c')^2 for all N
  cm2 = cm.^2;
  dL = sum (reshape (dMdc .* (cm2 (outidx_left, :)), [D, D, N]), 1);
  dL = reshape (dL, D, N); % sometimes Matlab adds a dimension of size 1
  dL = dL ./ repmat (sum (Wt, 1).^2, D, 1);
  ymm = -0.5 * dL ./ Ms;
  
  y = [cm; ymm];

end % function


