%% A simple demo illustrating how to use the code.

%% Copyright 2012 Soren Hauberg
%%
%% This program is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program.  If not, see <http://www.gnu.org/licenses/>.

clear all
close all

%% Generate synthetic data
D = 2;
X1 = rand (D, 100);
X1 (1, :) = X1 (1, :) - 0.55;
X2 = rand (D, 100);
X2 (1, :) = X2 (1, :) + 0.55;
[data, labels] = cat_with_labels (X1, X2);

%% Create manifold
x_r = {[0, 0.5], [0.5, 0.5], [1, 0.5]};
M_r = {0.5*eye(D), diag([7.5, 0.25]), 0.5*eye(D)};
R = numel (x_r);
manifold = create_manifold (x_r, M_r, 'diagonal', 10);

%% Compute mean value, data variance and logarithmic maps of the data
[mu, sigma, Logs] = ml_mean (data.', manifold);

%% Plot data and metric tensors
figure ('Name', 'Data and Metric Tensors');
h = plot (data (1, labels==1), data (2, labels==1), 'ro', data (1, labels==2), data (2, labels==2), 'bo');
set (h (1), 'markerfacecolor', 'r');
set (h (2), 'markerfacecolor', 'b');
axis equal
hold on
for r = 1:R
  plot_metric (M_r{r}, x_r{r}, 0.25*eye(D), 'k', 'linewidth', 2);
end % for
hold off

%% Plot trace of manifold tensor
figure ('Name', 'Trace of Metric Tensor and Data');
lim = linspace (-0.5, 1.5, 500);
trace_metric_image (manifold, lim);
hold on
  h = plot (data (1, labels==1), data (2, labels==1), 'ro', data (1, labels==2), data (2, labels==2), 'bo');
  set (h (1), 'markerfacecolor', 'r');
  set (h (2), 'markerfacecolor', 'b');
hold off

%% Plot logarithmic maps of data
figure ('Name', 'Data in the Tangent Space at the Mean');
h = plot (Logs (labels==1, 1), Logs (labels==1, 2), 'ro', Logs (labels==2, 1), Logs (labels==2, 2), 'bo');
set (h (1), 'markerfacecolor', 'r');
set (h (2), 'markerfacecolor', 'b');
axis equal
