function J = numerical_jacobian (state, manifold, delta)
  %% Check input
  if (nargin < 2)
    error ('numerical_jacobian: not enough inpurt arguments');
  end % if
  if (nargin < 3)
    delta = 1e-6;
  end % if
  
  %% Compute Jacobian based on finite differences
  geo_fun = @(s) manifold.geo_fun (s, manifold);
  F0 = geo_fun (state);
  D = size (state, 1);
  if (strcmpi (manifold.geo_options.Vectorized, 'On'))
    state_D = repmat (state, 1, D) + delta * eye (D);
    FD = geo_fun (state_D);
    J = (FD - repmat (F0, 1, D)) ./ delta;
  else
    J = NaN (D, D);
    for d = 1:D
      state_d = state;
      state_d (d) = state_d (d) + delta;
      Fd = geo_fun (state_d);
      J (:, d) = (Fd - F0) ./ delta;
    end % for
  end % if
end % function
