% function plot_metric (M, mu, basis, varargin)
%   Plot a 2D metric tensor as an ellipse.
%
%   Input:
%     M:     a metric tensor.
%     mu:    the location of the metric tensor.
%     basis: a change of basis applied to the metric tensor; if in doubt
%            just use the identity matrix.
%   Further input arguments are passed to the 'plot' function.

%% Copyright 2012 Soren Hauberg
%%
%% This program is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program.  If not, see <http://www.gnu.org/licenses/>.

function plot_metric (M, mu, basis, varargin)
  t = linspace (0, 2*pi, 100);
  xy = [cos(t); sin(t)];
  C = chol (basis.' * pinv (M) * basis);
  E = C * xy + repmat (mu(:), 1, size (xy, 2));
  plot (E (1, :), E (2, :), varargin {:})
end % function
